@@ -29,6 +29,7 @@ import java.util.Collection;
 import java.util.Date;
 import org.apache.guacamole.GuacamoleException;
 import org.apache.guacamole.auth.jdbc.base.RestrictedObject;
+import org.apache.guacamole.auth.jdbc.JDBCEnvironment;
 import org.apache.guacamole.auth.jdbc.activeconnection.ActiveConnectionDirectory;
 import org.apache.guacamole.auth.jdbc.base.ActivityRecordModel;
 import org.apache.guacamole.auth.jdbc.connection.ConnectionRecordSet;
@@ -50,6 +51,8 @@ import org.apache.guacamole.net.auth.SharingProfile;
 import org.apache.guacamole.net.auth.User;
 import org.apache.guacamole.net.auth.UserContext;
 import org.apache.guacamole.net.auth.UserGroup;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * UserContext implementation which is driven by an arbitrary, underlying
@@ -58,6 +61,11 @@ import org.apache.guacamole.net.auth.UserGroup;
 public class ModeledUserContext extends RestrictedObject
     implements org.apache.guacamole.net.auth.UserContext {
 
+    /**
+     * Logger for this class.
+     */
+    private static final Logger logger = LoggerFactory.getLogger(ModeledUserContext.class);
+
     /**
      * User directory restricted by the permissions of the user associated
      * with this context.
@@ -130,6 +138,12 @@ public class ModeledUserContext extends RestrictedObject
     @Inject
     private UserRecordMapper userRecordMapper;
 
+    /**
+     * The environment of the Guacamole server.
+     */
+    @Inject
+    private JDBCEnvironment environment;
+
     /**
      * The activity record associated with this user's Guacamole session. If
      * this user's session will not have an associated activity record, such as
@@ -162,8 +176,12 @@ public class ModeledUserContext extends RestrictedObject
      * invocation has any effect. If this function is never invoked, no
      * activity record will be recorded, including when this UserContext is
      * invalidated.
+     * 
+     * @throws GuacamoleException
+     *     If an error occurs retrieving configuration information from the
+     *     environment.
      */
-    public void recordUserLogin() {
+    public void recordUserLogin() throws GuacamoleException {
 
         // Do nothing if invoked multiple times
         if (userRecord != null)
@@ -176,7 +194,7 @@ public class ModeledUserContext extends RestrictedObject
         userRecord.setRemoteHost(getCurrentUser().getCredentials().getRemoteAddress());
 
         // Insert record representing login
-        userRecordMapper.insert(userRecord);
+        userRecordMapper.insert(userRecord, environment.getCaseSensitivity());
         
     }
 
@@ -296,4 +314,31 @@ public class ModeledUserContext extends RestrictedObject
 
     }
 
+    @Override
+    public boolean isValid() {
+
+        try {
+            // If access window enforcement is disabled for active sessions,
+            // skip validity checks entirely
+            if (!environment.enforceAccessWindowsForActiveSessions())
+                return true;
+        }
+
+        catch (GuacamoleException e) {
+
+            logger.warn(
+                    "Unable to determine if access window enforcement is"
+                    + " enabled for active sessions; enforcing by default: {}"
+                    , e.getMessage());
+            logger.debug("Unable to determine access window enforcement policy.", e);
+
+        }
+
+        // A user context is valid if the associated user's account is valid
+        // for the current date, and the user is within an access time window
+        ModeledUser user = getCurrentUser().getUser();
+        return user.isAccountValid() && user.isAccountAccessible();
+
+    }
+
 }
