@@ -32,8 +32,18 @@ var Guacamole = Guacamole || {};
  * @param {!Blob|Guacamole.Tunnel} source
  *     The Blob from which the instructions of the recording should
  *     be read.
+ * @param {number} [refreshInterval=1000]
+ *     The minimum number of milliseconds between updates to the recording
+ *     position through the provided onseek() callback. If non-positive, this
+ *     parameter will be ignored, and the recording position will only be
+ *     updated when seek requests are made, or when new frames are rendered.
+ *     If not specified, refreshInterval will default to 1000 milliseconds.
  */
-Guacamole.SessionRecording = function SessionRecording(source) {
+Guacamole.SessionRecording = function SessionRecording(source, refreshInterval) {
+
+    // Default the refresh interval to 1 second if not specified otherwise
+    if (refreshInterval === undefined)
+        refreshInterval = 1000;
 
     /**
      * Reference to this Guacamole.SessionRecording.
@@ -139,13 +149,13 @@ Guacamole.SessionRecording = function SessionRecording(source) {
     var currentFrame = -1;
 
     /**
-     * The timestamp of the frame when playback began, in milliseconds. If
+     * The position of the recording when playback began, in milliseconds. If
      * playback is not in progress, this will be null.
      *
      * @private
      * @type {number}
      */
-    var startVideoTimestamp = null;
+    var startVideoPosition = null;
 
     /**
      * The real-world timestamp when playback began, in milliseconds. If
@@ -156,6 +166,14 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     var startRealTimestamp = null;
 
+    /**
+     * The current position within the recording, in milliseconds.
+     *
+     * @private
+     * @type {!number}
+     */
+    var currentPosition = 0;
+
     /**
      * An object containing a single "aborted" property which is set to
      * true if the in-progress seek operation should be aborted. If no seek
@@ -211,6 +229,25 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     var seekCallback = null;
 
+    /**
+     * Any current timeout associated with scheduling frame replay, or updating
+     * the current position, or null if no frame position increment is currently
+     * scheduled.
+     *
+     * @private
+     * @type {number}
+     */
+    var updateTimeout = null;
+
+    /**
+     * The browser timestamp of the last time that currentPosition was updated
+     * while playing, or null if the recording is not currently playing.
+     *
+     * @private
+     * @type {number}
+     */
+    var lastUpdateTimestamp = null;
+
     /**
      * Parses all Guacamole instructions within the given blob, invoking
      * the provided instruction callback for each such instruction. Once
@@ -338,6 +375,28 @@ Guacamole.SessionRecording = function SessionRecording(source) {
     // Hide cursor unless mouse position is received
     playbackClient.getDisplay().showCursor(false);
 
+    /**
+     * A key event interpreter to split all key events in this recording into
+     * human-readable batches of text. Constrcution is deferred until the first
+     * event is processed, to enable recording-relative timestamps.
+     *
+     * @type {!Guacamole.KeyEventInterpreter}
+     */
+    var keyEventInterpreter = null;
+
+    /**
+     * Initialize the key interpreter. This function should be called only once
+     * with the first timestamp in the recording as an argument.
+     *
+     * @private
+     * @param {!number} startTimestamp
+     *     The timestamp of the first frame in the recording, i.e. the start of
+     *     the recording.
+     */
+    function initializeKeyInterpreter(startTimestamp) {
+        keyEventInterpreter = new Guacamole.KeyEventInterpreter(startTimestamp);
+    }
+
     /**
      * Handles a newly-received instruction, whether from the main Blob or a
      * tunnel, adding new frames and keyframes as necessary. Load progress is
@@ -369,6 +428,11 @@ Guacamole.SessionRecording = function SessionRecording(source) {
             frames.push(frame);
             frameStart = frameEnd;
 
+            // If this is the first frame, intialize the key event interpreter
+            // with the timestamp of the first frame
+            if (frames.length === 1)
+                initializeKeyInterpreter(timestamp);
+
             // This frame should eventually become a keyframe if enough data
             // has been processed and enough recording time has elapsed, or if
             // this is the absolute first frame
@@ -376,6 +440,7 @@ Guacamole.SessionRecording = function SessionRecording(source) {
                     && timestamp - frames[lastKeyframe].timestamp >= KEYFRAME_TIME_INTERVAL)) {
                 frame.keyframe = true;
                 lastKeyframe = frames.length - 1;
+
             }
 
             // Notify that additional content is available
@@ -384,6 +449,8 @@ Guacamole.SessionRecording = function SessionRecording(source) {
 
         }
 
+        else if (opcode === 'key')
+            keyEventInterpreter.handleKeyEvent(args);
     };
 
     /**
@@ -398,8 +465,10 @@ Guacamole.SessionRecording = function SessionRecording(source) {
     };
 
     // Read instructions from provided blob, extracting each frame
-    if (source instanceof Blob)
+    if (source instanceof Blob) {
+        recordingBlob = source;
         parseBlob(recordingBlob, loadInstruction, notifyLoaded);
+    }
 
     // If tunnel provided instead of Blob, extract frames, etc. as instructions
     // are received, buffering things into a Blob for future seeks
@@ -448,6 +517,11 @@ Guacamole.SessionRecording = function SessionRecording(source) {
                     instructionBuffer = '';
                 }
 
+                // Now that the recording is fully processed, and all key events
+                // have been extracted, call the onkeyevents handler if defined
+                if (recording.onkeyevents)
+                    recording.onkeyevents(keyEventInterpreter.getEvents());
+
                 // Consider recording loaded if tunnel has closed without errors
                 if (!errorEncountered)
                     notifyLoaded();
@@ -480,8 +554,9 @@ Guacamole.SessionRecording = function SessionRecording(source) {
     };
 
     /**
-     * Searches through the given region of frames for the frame having a
-     * relative timestamp closest to the timestamp given.
+     * Searches through the given region of frames for the closest frame
+     * having a relative timestamp less than or equal to the to the given
+     * relative timestamp.
      *
      * @private
      * @param {!number} minIndex
@@ -502,9 +577,22 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     var findFrame = function findFrame(minIndex, maxIndex, timestamp) {
 
-        // Do not search if the region contains only one element
-        if (minIndex === maxIndex)
-            return minIndex;
+        // The region has only one frame - determine if it is before or after
+        // the requested timestamp
+        if (minIndex === maxIndex) {
+
+            // Skip checking if this is the very first frame - no frame could
+            // possibly be earlier
+            if (minIndex === 0)
+                return minIndex;
+
+            // If the closest frame occured after the requested timestamp,
+            // return the previous frame, which will be the closest with a
+            // timestamp before the requested timestamp
+            if (toRelativeTimestamp(frames[minIndex].timestamp) > timestamp)
+                return minIndex - 1;
+
+        }
 
         // Split search region into two halves
         var midIndex = Math.floor((minIndex + maxIndex) / 2);
@@ -597,10 +685,11 @@ Guacamole.SessionRecording = function SessionRecording(source) {
         // Replay any applicable incremental frames
         var continueReplay = function continueReplay() {
 
-            // Notify of changes in position
+            // Set the current position and notify changes
             if (recording.onseek && currentFrame > startIndex) {
-                recording.onseek(toRelativeTimestamp(frames[currentFrame].timestamp),
-                    currentFrame - startIndex, index - startIndex);
+                currentPosition = toRelativeTimestamp(frames[currentFrame].timestamp);
+                recording.onseek(currentPosition, currentFrame - startIndex,
+                        index - startIndex);
             }
 
             // Cancel seek if aborted
@@ -621,8 +710,18 @@ Guacamole.SessionRecording = function SessionRecording(source) {
         // immediately if no delay was requested
         var continueAfterRequiredDelay = function continueAfterRequiredDelay() {
             var delay = nextRealTimestamp ? Math.max(nextRealTimestamp - new Date().getTime(), 0) : 0;
-            if (delay)
-                window.setTimeout(continueReplay, delay);
+            if (delay) {
+
+                // Clear any already-scheduled update before scheduling again
+                // to avoid multiple updates in flight at the same time
+                updateTimeout && clearTimeout(updateTimeout);
+
+                // Schedule with the appropriate delay
+                updateTimeout = window.setTimeout(function timeoutComplete() {
+                    updateTimeout = null;
+                    continueReplay();
+                }, delay);
+            }
             else
                 continueReplay();
         };
@@ -651,7 +750,7 @@ Guacamole.SessionRecording = function SessionRecording(source) {
         }
 
         continueAfterRequiredDelay();
-        
+
     };
 
     /**
@@ -676,20 +775,73 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     var continuePlayback = function continuePlayback() {
 
+        // Do not continue playback if the recording is paused
+        if (!recording.isPlaying())
+            return;
+
         // If frames remain after advancing, schedule next frame
         if (currentFrame + 1 < frames.length) {
 
             // Pull the upcoming frame
             var next = frames[currentFrame + 1];
 
-            // Calculate the real timestamp corresponding to when the next
-            // frame begins
-            var nextRealTimestamp = next.timestamp - startVideoTimestamp + startRealTimestamp;
+            // The number of elapsed milliseconds on the clock since playback began
+            var realLifePlayTime = Date.now() - startRealTimestamp;
 
-            // Advance to next frame after enough time has elapsed
-            seekToFrame(currentFrame + 1, function frameDelayElapsed() {
-                continuePlayback();
-            }, nextRealTimestamp);
+            // The number of milliseconds between the recording position when
+            // playback started and the position of the next frame
+            var timestampOffset = (
+                    toRelativeTimestamp(next.timestamp) - startVideoPosition);
+
+            // The delay until the next frame should be rendered, taking into
+            // account any accumulated delays from rendering frames so far
+            var nextFrameDelay = timestampOffset - realLifePlayTime;
+
+            // The delay until the refresh interval would induce an update to
+            // the current recording position, rounded to the nearest whole
+            // multiple of refreshInterval to ensure consistent timing for
+            // refresh intervals even with inconsistent frame timing
+            var nextRefreshDelay = refreshInterval >= 0
+                    ? (refreshInterval * (Math.floor(
+                        (currentPosition + refreshInterval) / refreshInterval))
+                    ) - currentPosition
+                    : nextFrameDelay;
+
+            // If the next frame will occur before the next refresh interval,
+            // advance to the frame after the appropriate delay
+            if (nextFrameDelay <= nextRefreshDelay)
+
+                seekToFrame(currentFrame + 1, function frameDelayElapsed() {
+
+                    // Record when the timestamp was updated and continue on
+                    lastUpdateTimestamp = Date.now();
+                    continuePlayback();
+
+                }, Date.now() + nextFrameDelay);
+
+            // The position needs to be incremented before the next frame
+            else {
+
+                // Clear any existing update timeout
+                updateTimeout && window.clearTimeout(updateTimeout);
+
+                updateTimeout = window.setTimeout(function incrementPosition() {
+
+                    updateTimeout = null;
+
+                    // Update the position
+                    currentPosition += nextRefreshDelay;
+
+                    // Notifiy the new position using the onseek handler
+                    if (recording.onseek)
+                        recording.onseek(currentPosition);
+
+                    // Record when the timestamp was updated and continue on
+                    lastUpdateTimestamp = Date.now();
+                    continuePlayback();
+
+                }, nextRefreshDelay);
+            }
 
         }
 
@@ -755,6 +907,17 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     this.onpause = null;
 
+    /**
+     * Fired with all extracted key events when the recording is fully
+     * processed. The callback will be invoked with an empty list
+     * if no key events were extracted.
+     *
+     * @event
+     * @param {!Guacamole.KeyEventInterpreter.KeyEvent[]} batch
+     *     The extracted key events.
+     */
+    this.onkeyevents = null;
+
     /**
      * Fired whenever the playback position within the recording changes.
      *
@@ -837,7 +1000,7 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      *     true if playback is currently in progress, false otherwise.
      */
     this.isPlaying = function isPlaying() {
-        return !!startVideoTimestamp;
+        return !!startRealTimestamp;
     };
 
     /**
@@ -849,13 +1012,7 @@ Guacamole.SessionRecording = function SessionRecording(source) {
      */
     this.getPosition = function getPosition() {
 
-        // Position is simply zero if playback has not started at all
-        if (currentFrame === -1)
-            return 0;
-
-        // Return current position as a millisecond timestamp relative to the
-        // start of the recording
-        return toRelativeTimestamp(frames[currentFrame].timestamp);
+        return currentPosition;
 
     };
 
@@ -899,11 +1056,11 @@ Guacamole.SessionRecording = function SessionRecording(source) {
 
             // Store timestamp of playback start for relative scheduling of
             // future frames
-            var next = frames[currentFrame + 1];
-            startVideoTimestamp = next.timestamp;
-            startRealTimestamp = new Date().getTime();
+            startVideoPosition = currentPosition;
+            startRealTimestamp = Date.now();
 
             // Begin playback of video
+            lastUpdateTimestamp = Date.now();
             continuePlayback();
 
         }
@@ -955,8 +1112,23 @@ Guacamole.SessionRecording = function SessionRecording(source) {
 
         };
 
-        // Perform seek
-        seekToFrame(findFrame(0, frames.length - 1, position), seekCallback);
+        // Find the index of the closest frame at or before the requested position
+        var closestFrame = findFrame(0, frames.length - 1, position);
+
+        // Seek to the closest frame before or at the requested position
+        seekToFrame(closestFrame, function seekComplete() {
+
+            // Update the current position to the requested position
+            // and invoke the the onseek callback. Note that this is the
+            // position provided to this function, NOT the position of the
+            // frame that was just seeked
+            currentPosition = position;
+            if (recording.onseek)
+                recording.onseek(position);
+
+            seekCallback();
+
+        });
 
     };
 
@@ -986,6 +1158,13 @@ Guacamole.SessionRecording = function SessionRecording(source) {
         // Abort any in-progress seek / playback
         abortSeek();
 
+        // Cancel any currently-scheduled updates
+        updateTimeout && clearTimeout(updateTimeout);
+
+        // Increment the current position by the amount of time passed since the
+        // the last time it was updated
+        currentPosition += Date.now() - lastUpdateTimestamp;
+
         // Stop playback only if playback is in progress
         if (recording.isPlaying()) {
 
@@ -994,7 +1173,8 @@ Guacamole.SessionRecording = function SessionRecording(source) {
                 recording.onpause();
 
             // Playback is stopped
-            startVideoTimestamp = null;
+            lastUpdateTimestamp = null;
+            startVideoPosition = null;
             startRealTimestamp = null;
 
         }
@@ -1122,4 +1302,4 @@ Guacamole.SessionRecording._PlaybackTunnel = function _PlaybackTunnel() {
             tunnel.oninstruction(opcode, args);
     };
 
-};
\ No newline at end of file
+};
