@@ -127,6 +127,11 @@ public class DelegatingUserContext implements UserContext {
         return userContext.getUserAttributes();
     }
 
+    @Override
+    public Collection<Form> getUserPreferenceAttributes() {
+        return userContext.getUserPreferenceAttributes();
+    }
+
     @Override
     public Collection<Form> getUserGroupAttributes() {
         return userContext.getUserGroupAttributes();
@@ -157,4 +162,9 @@ public class DelegatingUserContext implements UserContext {
         return userContext.getPrivileged();
     }
 
+    @Override
+    public boolean isValid() {
+        return userContext.isValid();
+    }
+
 }
