@@ -31,7 +31,7 @@ import org.apache.guacamole.net.auth.Permissions;
 import org.apache.guacamole.net.auth.permission.ObjectPermission;
 import org.apache.guacamole.net.auth.permission.Permission;
 import org.apache.guacamole.net.auth.permission.SystemPermission;
-import org.apache.guacamole.rest.APIPatch;
+import org.apache.guacamole.rest.jsonpatch.APIPatch;
 
 /**
  * A REST resource which abstracts the operations available on the permissions
