@@ -31,6 +31,9 @@ public interface SessionResourceFactory {
      * Creates a new SessionResource which exposes the contents of the
      * given GuacamoleSession.
      *
+     * @param token
+     *     The authentication token associated with the given session.
+     *
      * @param session
      *     The GuacamoleSession whose contents should be exposed.
      *
@@ -38,6 +41,6 @@ public interface SessionResourceFactory {
      *     A new SessionResource which exposes the contents of the given
      *     GuacamoleSession.
      */
-    SessionResource create(GuacamoleSession session);
+    SessionResource create(String token, GuacamoleSession session);
 
 }
